var WizardEngine = function(renderEngine, templateManager, metadataFactory, eventFactory, eventHandler, eventEngine, MetaDataLoader) {
	var _bindEvents = function(owner, events) {
		eventEngine.bindEvents(
			owner,
			events,
			function() {
				return [];
			},
			function() {
				return templateManager.container.widgets[0].currentRow;
			},
			function() {
				return templateManager.container.widgets[0].selectedRow;
			}
		);
	};
	var _createStageEvents = function(wizard) {
		var stage = wizard.currentStage;
		var events = stage.events;
		_bindEvents(stage, events);
	};
	var _createWizardEvents = function(wizard) {
		_bindEvents(wizard, wizard.events);
	};
	var _executeEvent = function(eventCallback) {
		if (eventCallback) {
			var e = eventCallback();
			return e;
		}
	};
	var _chooseRoute = function(stage) {
		var event = _executeEvent(stage.onDecision);
		var routeName = event ? event.data : undefined;
		if (routeName) {
			var nextStages = stage.nextStages;
			for (var i in nextStages) {
				if (nextStages[i].route === routeName) {
					return nextStages[i].name;
				}
			}
		} else {
			return stage.nextStages[0].name;
		}
	};
	var _moveStage = function(wizard, stage) {
		return MetaDataLoader.loadContainer(stage.window).then(function() {
			renderEngine.changeContainer(stage.window);
			_executeEvent(wizard.currentStage.afterOpenStage);
			wizard.currentStage = stage;
		});
	};
	var _next = function(wizard) {
		var chosenRoute = _chooseRoute(wizard.currentStage);
		var stage = wizard.stages[chosenRoute];
		if (!templateManager.historyItems.length || templateManager.historyItems[templateManager.historyItems
				.length - 1] !== stage.name) {
			wizard.history.push(wizard.currentStage);
			templateManager.historyItems.push(stage.name);
		}
		return _moveStage(wizard, stage);
	};
	this.startWizard = function(wizard) {
		wizard.init();

		_createWizardEvents(wizard);
		_createStageEvents(wizard);
		// save the current window (container) to go back if user hits cancel...
		wizard.callerWindow = templateManager.container ? templateManager.container.name : "";
		templateManager.wizard = wizard;
		var containerName = templateManager.wizard.currentStage.window;
		MetaDataLoader.loadContainer(templateManager.wizard.currentStage.window).then(function() {
			renderEngine.changeContainer(containerName);
			_executeEvent(wizard.currentStage.afterOpenStage);
		});
	};
	var thenable = {
		then: function(fn) {
			if (fn) {
				fn();
			}
		}
	};
	this.next = function() {
		var ret = thenable;
		var wizard = templateManager.wizard;
		if (wizard.currentStage.nextStages && wizard.currentStage.nextStages.length > 0) {
			var event = _executeEvent(wizard.currentStage.beforeNextStage);
			if (!event || !event.isDefaultPrevented()) {
				// It must be done because the method "_next" updates the value of currentStage.
				var afterNextStageEvent = wizard.currentStage.afterNextStage;
				ret = _next(wizard).then(function() {
					++wizard.currentStageIndex;
					_createStageEvents(wizard);
					_executeEvent(afterNextStageEvent);
				});
			}
		} else {
			_executeEvent(wizard.currentStage.onFinish);
		}
		return ret;
	};
	this.previous = function() {
		var ret = thenable;
		var wizard = templateManager.wizard;
		if (wizard.history.length > 0) {
			var event = _executeEvent(wizard.currentStage.beforePreviousStage);
			if (!event || !event.isDefaultPrevented()) {
				var afterPreviousStageEvent = wizard.currentStage.afterPreviousStage;
				_createStageEvents(wizard);
				var previousStage = wizard.history.pop();
				templateManager.historyItems.pop();
				if (previousStage) {
					// It must be done because the method "_moveStage" updates the valu of currentStage.
					ret = _moveStage(wizard, previousStage).then(function() {
						--wizard.currentStageIndex;
						_executeEvent(afterPreviousStageEvent);
					});
				}
			}
		}
		return ret;
	};
};